:li
/********************************************************************
 *
 * Motif-Programm zur Linienausgabe
 *
 ********************************************************************/

/* Allgemeine Header-Dateien */
#include <stdio.h>
#include <stdlib.h>
#include <Xm/Xm.h>

/* Header-Dateien fuer verwendete Widgetklassen */
#include <Xm/DrawingA.h>

/* Klassenname der Applikation */
#define APPL_KLASSE    "XmBsp"

/* Vorwaerts-Deklarationen */
void initDrawingArea();
void redisplayDrawingAreaCB();
void clearDrawingAreaCB();
void exitCB();


main (argc, argv)
int   argc;
char* argv[];
{
    XtAppContext kontext;        /* Applikations-Umgebung */
    Widget       applShell;      /* Applikations-Shell */
    int		 linienBreite;   /* Linienbreite */

    /* Initialisierungen und Applikations-Shell erzeugen */
    applShell = XtAppInitialize (&kontext, APPL_KLASSE,
                                 (XrmOptionDescRec*)NULL, 0,
                                 &argc, argv, (String*)NULL,
                                 (Arg*)NULL, 0);

    /* (nicht verarbeiteter) Parameter ist Linienbreite */
    if (argc > 1) {
	linienBreite = atoi (argv[1]);
    }
    else {
	linienBreite = 0;	/* optimierte 1 */
    }

    /* Widgetbaum unter Applikations-Shell aufbauen */
    initDrawingArea (applShell, linienBreite);

    /* Widgetbaum realisieren */
    XtRealizeWidget (applShell);

    /* Hauptschleife mit Ereignisbehandlung */
    XtAppMainLoop (kontext);
}


void initDrawingArea (elternWidget, linienBreite)
Widget elternWidget;
int linienBreite;
{
    Widget da;		/* XmDrawingArea-Widget */
    GC gc;		/* Graphics-Context */
    XGCValues gcv;	/* Werte-Struktur dafuer */
    Pixel fg;		/* Vordergrundfarbe */
    Arg args[10];
    Cardinal n;

    /* XmDrawingArea-Widget erzeugen */
    da = XtCreateManagedWidget ("box", xmDrawingAreaWidgetClass,
				elternWidget, (Arg*)NULL, 0);

    /* Vordergrundfarbe abfragen */
    n = 0;
    XtSetArg (args[n], XmNforeground, &fg);  n++;
    XtGetValues (da, args, n);

    /* Graphics-Context dazu erzeugen */
    gcv.foreground = fg;
    gcv.line_width = linienBreite;
    gc = XtGetGC (da, GCForeground | GCLineWidth, &gcv);

    /* Callback zum Ausgeben des Fensterinhalts installieren
     * (Linien sternfoermig ausgeben)
     */
    XtAddCallback (da, XmNexposeCallback,
		   redisplayDrawingAreaCB, (XtPointer)gc);

    /* Callback zur Reaktion auf Groessenaenderung installieren
     * (bisherigen Fensterinhalt loeschen)
     */
    XtAddCallback (da, XmNresizeCallback,
		   clearDrawingAreaCB, (XtPointer)NULL);

    /* Exit-Callback zur Reaktion auf Eingaben installieren
     * (jede Maus- und Tastatur-Eingabe beendet das Programm)
     */
    XtAddCallback (da, XmNinputCallback,
		   exitCB, (XtPointer)NULL);
}

/* ARGSUSED */
void redisplayDrawingAreaCB (widget, clientDaten, aufrufDaten)
Widget    widget;      /* aufrufendes Widget (nicht verwendet) */
XtPointer clientDaten; /* Client-Daten (GC) */
XtPointer aufrufDaten; /* Aufruf-Daten (Callback-Informationen) */
{
    XmDrawingAreaCallbackStruct* evp;
    Dimension breite, hoehe;
    Arg args[10];
    Cardinal n;
    int x, y;

    /* nur der letzte einer Folge von Expose-Events ist von
     * Interesse, da der Inhalt immer komplett ausgegeben wird
     */
    evp = (XmDrawingAreaCallbackStruct*)aufrufDaten;
    if (evp->event->xexpose.count != 0) {
	return;
    }

    /* aktuelle Groesse abfragen */
    n = 0;
    XtSetArg (args[n], XmNwidth, &breite);  n++;
    XtSetArg (args[n], XmNheight, &hoehe);  n++;
    XtGetValues (widget, args, n);

    /* Linien sternfoermig ausgeben */
    for (x=breite; x>0; x-=10) {
	XDrawLine (XtDisplay(widget), XtWindow(widget), (GC)clientDaten,
		   0, hoehe, x, 0);
    }
    for (y=10; y<hoehe; y+=10) {
	XDrawLine (XtDisplay(widget), XtWindow(widget), (GC)clientDaten,
		   0, hoehe, breite, y);
    }
}

/* ARGSUSED */
void clearDrawingAreaCB (widget, clientDaten, aufrufDaten)
Widget    widget;      /* aufrufendes Widget (nicht verwendet) */
XtPointer clientDaten; /* Client-Daten (nicht verwendet) */
XtPointer aufrufDaten; /* Aufruf-Daten (nicht verwendet) */
{
    /* Fensterinhalt loeschen */
    if (XtIsRealized(widget)) {
        XClearWindow (XtDisplay(widget), XtWindow(widget));
    }
}

/* ARGSUSED */
void exitCB (widget, clientDaten, aufrufDaten)
Widget    widget;      /* aufrufendes Widget (nicht verwendet) */
XtPointer clientDaten; /* Client-Daten (nicht verwendet) */
XtPointer aufrufDaten; /* Aufruf-Daten (nicht verwendet) */
{
    /* Programm mit fehlerfreiem Status beenden */ 
    exit (0);
}



